@@ -14,6 +14,8 @@ from .redshift import Redshift
 from .presto import Presto
 from .databricks import Databricks
 from .trino import Trino
+from .clickhouse import Clickhouse
+from .vertica import Vertica
 
 
 @dataclass
@@ -85,6 +87,8 @@ MATCH_URI_PATH = {
         help_str="databricks://:access_token@server_name/http_path",
     ),
     "trino": MatchUriPath(Trino, ["catalog", "schema"], help_str="trino://<user>@<host>/<catalog>/<schema>"),
+    "clickhouse": MatchUriPath(Clickhouse, ["database?"], help_str="clickhouse://<user>:<pass>@<host>/<database>"),
+    "vertica": MatchUriPath(Vertica, ["database?"], help_str="vertica://<user>:<pass>@<host>/<database>"),
 }
 
 
@@ -110,6 +114,8 @@ def connect_to_uri(db_uri: str, thread_count: Optional[int] = 1) -> Database:
     - presto
     - databricks
     - trino
+    - clickhouse
+    - vertica
     """
 
     dsn = dsnparse.parse(db_uri)
@@ -197,9 +203,11 @@ def connect(db_conf: Union[str, dict], thread_count: Optional[int] = 1) -> Datab
     - presto
     - databricks
     - trino
+    - clickhouse
+    - vertica
     """
     if isinstance(db_conf, str):
         return connect_to_uri(db_conf, thread_count)
     elif isinstance(db_conf, dict):
         return connect_with_dict(db_conf, thread_count)
-    raise TypeError(db_conf)
+    raise TypeError(f"db configuration must be a URI string or a dictionary. Instead got '{db_conf}'.")
